@@ -28,8 +28,11 @@
 
 #include "drouter.h"
 #include "protocol.h"
+#include "scripthost.h"
 
-#define DROUTERD_USAGE "[options]\n"
+#define DROUTERD_SCRIPTS_DIRECTORY QString("/etc/drouter.d/scripts")
+#define DROUTERD_SCRIPTS_FILTER QString("*.py")
+#define DROUTERD_USAGE "[--no-scripts]\n"
 
 class MainObject : public QObject
 {
@@ -38,11 +41,14 @@ class MainObject : public QObject
   MainObject(QObject *parent=0);
 
  private slots:
+  void scriptsData();
   void signalData();
 
  private:
   QList<Protocol *> main_protocols;
+  QList<ScriptHost *> main_scripts;
   DRouter *main_drouter;
+  QTimer *main_script_timer;
   QTimer *main_signal_timer;
 };
 
