@@ -67,7 +67,14 @@ ProtocolD::ProtocolD(DRouter *router,int sock,QObject *parent)
 						   const QHostAddress &,int)),
 	  this,SLOT(processSetGpioCrosspointD(int,const QHostAddress &,int,
 					      const QHostAddress &,int)));
-
+  connect(d_server,SIGNAL(processSetGpoState(int,const QHostAddress &,int,
+					     const QString &)),
+	  this,SLOT(processSetGpoStateD(int,const QHostAddress &,int,
+					const QString &)));
+  connect(d_server,SIGNAL(processSetGpiState(int,const QHostAddress &,int,
+					     const QString &)),
+	  this,SLOT(processSetGpiStateD(int,const QHostAddress &,int,
+					const QString &)));
   d_server->setReady();
 }
 
@@ -391,6 +398,28 @@ void ProtocolD::processSetGpioCrosspointD(int id,
 }
 
 
+void ProtocolD::processSetGpoStateD(int id,const QHostAddress &gpo_hostaddr,
+				    int gpo_slot,const QString &code)
+{
+  SyLwrpClient *gpo_lwrp;
+
+  if((gpo_lwrp=router()->node(gpo_hostaddr))!=NULL) {
+    gpo_lwrp->setGpoCode(gpo_slot,code);
+  }
+}
+
+
+void ProtocolD::processSetGpiStateD(int id,const QHostAddress &gpi_hostaddr,
+				    int gpi_slot,const QString &code)
+{
+  SyLwrpClient *gpi_lwrp;
+
+  if((gpi_lwrp=router()->node(gpi_hostaddr))!=NULL) {
+    gpi_lwrp->setGpiCode(gpi_slot,code);
+  }
+}
+
+
 void ProtocolD::processAddedNode(const SyNode &node)
 {
   SyLwrpClient *lwrp=NULL;
@@ -476,6 +505,9 @@ void ProtocolD::processAddedNode(const SyNode &node)
 
 void ProtocolD::processAboutToBeRemovedNode(const SyNode &node)
 {
+  if(node.hostAddress().isNull()) {
+    return;  // Zombie node!
+  }
   QList<NetConnection *> conns=d_server->connections();
   for(int i=0;i<conns.size();i++) {
     if(conns.at(i)!=NULL) {
@@ -656,8 +688,20 @@ QString ProtocolD::AlarmRecord(const QString &keyword,
     ret+="UNKNOWN\t";
     break;
   }
-  ret+=QString().sprintf("%d\t",chan);
-  ret+=QString().sprintf("%d",state);
+  switch(chan) {
+  case 0:
+    ret+="LEFT\t";
+    break;
+
+  case 1:
+    ret+="RIGHT\t";
+    break;
+
+  default:
+    ret+="UNKNOWN\t";
+    break;
+  }
+  ret+=QString().sprintf("%d\t",state);
   ret+="\r\n";
 
   return ret;
